
#pragma once

template<class T> class TVertexBuffer : public vector<T>
{
public:
	TVertexBuffer()
	{
		VertexBuffer = NULL;
	}
	virtual ~TVertexBuffer()
	{
		clear();
		Release();
	}

	// InSize is the maximum size the vertex buffer will ever grow to.  Leaving it at zero will create a vertex buffer at this arrays current size.
	// Setting InSize to something larger than zero allows you to pad the vertex buffer.
	virtual bool Create( int InSize )
	{
		if( !InSize )	InSize = size();
		if( !InSize )	return 1;

		HRESULT hr = D3DDevice->CreateVertexBuffer( InSize*GetVertexSize(), D3DUSAGE_WRITEONLY | D3DUSAGE_DYNAMIC, GetFVF(), D3DPOOL_DEFAULT, &VertexBuffer );
		return (hr == D3D_OK);
	}
	virtual bool Upload()
	{
		if( !size() )	return 1;

		VOID* Buffer;
		 VertexBuffer->Lock( 0, size()*GetVertexSize(), (byte**)&Buffer, D3DLOCK_DISCARD );
		::memcpy( (T*)Buffer, begin(), size()*GetVertexSize() );
		VertexBuffer->Unlock();

		return 1;
	}
	virtual void Release()
	{
		if( VertexBuffer )
			VertexBuffer->Release();
		VertexBuffer = NULL;
	}

	inline int GetVertexSize()
	{
		return sizeof(T);
	}
	inline DWORD GetFVF()
	{
		return T::FVF;
	}
	void SetPrimType( D3DPRIMITIVETYPE InPrimType )
	{
		PrimType = InPrimType;
	}
	virtual int GetPrimCount()
	{
		switch( PrimType )
		{
			case D3DPT_TRIANGLELIST:	return size()/3;
			case D3DPT_TRIANGLEFAN:		return size()-2;
			case D3DPT_TRIANGLESTRIP:	return size()-2;
			case D3DPT_LINESTRIP:		return size()-1;
			case D3DPT_LINELIST:		return size()/2;
			case D3DPT_POINTLIST:		return size();
		}

		_ASSERT(0);	// Unknown primitive type
		return 0;
	}

	LPDIRECT3DVERTEXBUFFER8 VertexBuffer;
	D3DPRIMITIVETYPE PrimType;
};
